# vim: set fileencoding=utf-8 :
"""
Authentication backends for role system


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement

from permission.models import Role
from permission.utils import permission_to_perm

__all__ = ('RoleBackend',)


class RoleBackend(object):
    """Authentication backend for role system"""
    supports_object_permissions = False
    supports_anonymous_user = False
    supports_inactive_user = True

    def authenticate(self, username, password):
        """This backend is only for checking permission"""
        return None

    def get_all_roles(self, user_obj):
        if hasattr(user_obj, '_role_cache'):
            return user_obj._role_cache
        roles = Role.objects.filter_by_user(user_obj)
        roles = set([r.codename for r in roles])
        user_obj._role_cache = roles
        return user_obj._role_cache

    def get_all_permissions(self, user_obj, obj=None):
        if obj is not None:
            # role permission system doesn't handle
            # object permission
            return set()
        if hasattr(user_obj, '_role_perm_cache'):
            return user_obj._role_perm_cache
        perms = Role.objects.get_all_permissions_of_user(user_obj)
        perms = set([permission_to_perm(p) for p in perms])
        user_obj._role_perm_cache = perms
        return user_obj._role_perm_cache

    def has_role(self, user_obj, role):
        if user_obj.is_anonymous() or not user_obj.is_active:
            return False
        roles = self.get_all_roles(user_obj)
        return role in roles

    def has_perm(self, user_obj, perm, obj=None):
        if user_obj.is_anonymous() or not user_obj.is_active:
            return False
        permissions = self.get_all_permissions(user_obj, obj)
        return perm in permissions

    def has_module_perms(self, user_obj, app_label):
        if not user_obj.is_active:
            return False
        permissions = self.get_all_permissions(user_obj)
        for perm in permissions:
            if perm[:perm.index('.')] == app_label:
                return True
        return False



# vim: set fileencoding=utf-8 :
"""
Unittest module of ...


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from mock import MagicMock as Mock
from django.test import TestCase
from django.http import HttpRequest
from django.views.generic import View

from permission.decorators import permission_required
from permission.decorators import class_decorators 
from permission.decorators import method_decorators 
from permission.decorators import function_decorators 
from permission.tests.models import Article

def viewbase(request, *args, **kwargs):
    assert isinstance(request, HttpRequest), \
        'An instance of "HttpRequest" is expected but "%s" is passed' % request

class ViewBase(View):
    def dispatch(self, request, *args, **kwargs):
        assert isinstance(self, ViewBase), \
            'An instance of "ViewBase" is expected but "%s" is passed' % self
        assert isinstance(request, HttpRequest), \
            'An instance of "HttpRequest" is expected but "%s" is passed' % request
    def get_object(self, queryset=None):
        return Article.objects.get(pk=1)

permission_required = permission_required('permission.add_article')
class_permission_required = class_decorators.permission_required('permission.add_article')
method_permission_required = method_decorators.permission_required('permission.add_article')
function_permission_required = function_decorators.permission_required('permission.add_article')

class PermissionDecoratorsTestCase(TestCase):

    fixtures = ('django_permission_test_datas.yaml',)

    def setUp(self):
        self.mock_request = Mock(spec=HttpRequest)
        self.mock_request.META = Mock()
        self.mock_request.user = Mock()
        self.mock_request.user.is_active.return_value = True
        self.mock_request.user.is_authenticated.return_value = True

    def test_function_views(self):
        # class_decorators cannot handle
        self.assertRaises(
                AttributeError,
                class_permission_required,
                viewbase
            )
        # method_decorators can handle function as well
        method_view = method_permission_required(viewbase)
        method_view(
                self.mock_request, 
                queryset=Article.objects.all(),
                object_id=1,
            )

        # function_decorators of course can handle
        function_view = function_permission_required(viewbase)
        function_view(
                self.mock_request, 
                queryset=Article.objects.all(),
                object_id=1,
            )

    def test_method_views(self):
        # class_decorators cannot handle
        self.assertRaises(
                AttributeError,
                class_permission_required,
                ViewBase.dispatch
            )
        # method_decorators of course can handle
        method_view = method_permission_required(ViewBase.dispatch)
        method_view(ViewBase(), self.mock_request, pk=1)

        # function_decorators cannot handle
        function_view = function_permission_required(ViewBase.dispatch)
        self.assertRaises(
                AttributeError, function_view,
                ViewBase(), self.mock_request, pk=1,
            )

    def test_class_views(self):
        # class view of course can handle
        class_view = class_permission_required(ViewBase)
        class_view.as_view()(self.mock_request, pk=1)

        # method_decorators cannot handle
        method_view = method_permission_required(ViewBase)
        self.assertFalse(hasattr(method_view, 'as_view'))

        # function_decorators cannot handle
        function_view = function_permission_required(ViewBase)
        self.assertFalse(hasattr(method_view, 'as_view'))

    def test_permission_required(self):
        # function
        functional_view = permission_required(viewbase)
        functional_view(
                self.mock_request, 
                queryset=Article.objects.all(),
                object_id=1,
            )

        # method
        method_view = permission_required(ViewBase.dispatch)
        method_view(ViewBase(), self.mock_request, pk=1)

        # class
        class_view = permission_required(ViewBase)
        class_view.as_view()(self.mock_request, pk=1)



# vim: set fileencoding=utf-8 :
"""
Unittest module of ...


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.test import TestCase

from permission.backends import RoleBackend
from permission.utils import permission_to_perm
from permission.tests import create_user
from permission.tests import create_role
from permission.tests import create_permission

class PermissionRoleBackendTestCase(TestCase):

    def setUp(self):
        # role1           -- user1, user2 -- perm1, perm2
        #   +- role2      -- user3        -- perm3
        #   +- role3      -- user4, user5 -- perm4, perm5
        #   |    +- role4 -- user6        -- perm6
        #   |    +- role5 -- user7        -- perm7
        #   +- role6      -- user8        -- perm8
        self.user1 = user1 = create_user('permission_test_user1')
        self.user2 = user2 = create_user('permission_test_user2')
        self.user3 = user3 = create_user('permission_test_user3')
        self.user4 = user4 = create_user('permission_test_user4')
        self.user5 = user5 = create_user('permission_test_user5')
        self.user6 = user6 = create_user('permission_test_user6')
        self.user7 = user7 = create_user('permission_test_user7')
        self.user8 = user8 = create_user('permission_test_user8')
        self.perm1 = perm1 = create_permission('permission_test_perm1')
        self.perm2 = perm2 = create_permission('permission_test_perm2')
        self.perm3 = perm3 = create_permission('permission_test_perm3')
        self.perm4 = perm4 = create_permission('permission_test_perm4')
        self.perm5 = perm5 = create_permission('permission_test_perm5')
        self.perm6 = perm6 = create_permission('permission_test_perm6')
        self.perm7 = perm7 = create_permission('permission_test_perm7')
        self.perm8 = perm8 = create_permission('permission_test_perm8')
        self.role1 = role1 = create_role('permission_test_role1')
        self.role2 = role2 = create_role('permission_test_role2', role1)
        self.role3 = role3 = create_role('permission_test_role3', role1)
        self.role4 = role4 = create_role('permission_test_role4', role3)
        self.role5 = role5 = create_role('permission_test_role5', role3)
        self.role6 = role6 = create_role('permission_test_role6', role1)
        role1._users.add(user1, user2)
        role2._users.add(user3)
        role3._users.add(user4, user5)
        role4._users.add(user6)
        role5._users.add(user7)
        role6._users.add(user8)
        role1._permissions.add(perm1, perm2)
        role2._permissions.add(perm3)
        role3._permissions.add(perm4, perm5)
        role4._permissions.add(perm6)
        role5._permissions.add(perm7)
        role6._permissions.add(perm8)

    def test_attributes_required(self):
        backend = RoleBackend()

        self.assertTrue(hasattr(backend, 'supports_object_permissions'))
        self.assertFalse(getattr(backend, 'supports_object_permissions'))

        self.assertTrue(hasattr(backend, 'supports_anonymous_user'))
        self.assertFalse(getattr(backend, 'supports_anonymous_user'))

        self.assertTrue(hasattr(backend, 'supports_inactive_user'))
        self.assertTrue(getattr(backend, 'supports_inactive_user'))

        self.assertTrue(hasattr(backend, 'authenticate'))
        self.assertEqual(backend.authenticate(None, None), None)

        self.assertTrue(hasattr(backend, 'has_perm'))
        self.assertTrue(hasattr(backend, 'has_role'))
        self.assertTrue(hasattr(backend, 'get_all_roles'))

    def test_get_all_roles(self):
        backend = RoleBackend()
        
        user1, user2, user3, user4 = self.user1, self.user2, self.user3, self.user4
        user5, user6, user7, user8 = self.user5, self.user6, self.user7, self.user8
        role1, role2, role3 = self.role1, self.role2, self.role3
        role4, role5, role6 = self.role4, self.role5, self.role6

        role1 = role1.codename
        role2 = role2.codename
        role3 = role3.codename
        role4 = role4.codename
        role5 = role5.codename
        role6 = role6.codename

        self.assertItemsEqual(backend.get_all_roles(user1), [
                role1,
            ])
        self.assertItemsEqual(backend.get_all_roles(user2), [
                role1,
            ])
        self.assertItemsEqual(backend.get_all_roles(user3), [
                role1, role2,
            ])
        self.assertItemsEqual(backend.get_all_roles(user4), [
                role1, role3,
            ])
        self.assertItemsEqual(backend.get_all_roles(user5), [
                role1, role3,
            ])
        self.assertItemsEqual(backend.get_all_roles(user6), [
                role1, role3, role4,
            ])
        self.assertItemsEqual(backend.get_all_roles(user7), [
                role1, role3, role5,
            ])
        self.assertItemsEqual(backend.get_all_roles(user8), [
                role1, role6,
            ])

    def test_get_all_permissions(self):
        backend = RoleBackend()
        
        user1, user2, user3, user4 = self.user1, self.user2, self.user3, self.user4
        user5, user6, user7, user8 = self.user5, self.user6, self.user7, self.user8
        perm1, perm2, perm3, perm4 = self.perm1, self.perm2, self.perm3, self.perm4
        perm5, perm6, perm7, perm8 = self.perm5, self.perm6, self.perm7, self.perm8

        perm1 = permission_to_perm(perm1)
        perm2 = permission_to_perm(perm2)
        perm3 = permission_to_perm(perm3)
        perm4 = permission_to_perm(perm4)
        perm5 = permission_to_perm(perm5)
        perm6 = permission_to_perm(perm6)
        perm7 = permission_to_perm(perm7)
        perm8 = permission_to_perm(perm8)

        self.assertItemsEqual(backend.get_all_permissions(user1), [
                perm1, perm2,
            ])
        self.assertItemsEqual(backend.get_all_permissions(user2), [
                perm1, perm2,
            ])
        self.assertItemsEqual(backend.get_all_permissions(user3), [
                perm1, perm2, perm3,
            ])
        self.assertItemsEqual(backend.get_all_permissions(user4), [
                perm1, perm2, perm4, perm5,
            ])
        self.assertItemsEqual(backend.get_all_permissions(user5), [
                perm1, perm2, perm4, perm5,
            ])
        self.assertItemsEqual(backend.get_all_permissions(user6), [
                perm1, perm2, perm4, perm5, perm6,
            ])
        self.assertItemsEqual(backend.get_all_permissions(user7), [
                perm1, perm2, perm4, perm5, perm7,
            ])
        self.assertItemsEqual(backend.get_all_permissions(user8), [
                perm1, perm2, perm8,
            ])

    def test_has_role(self):
        backend = RoleBackend()

        user1, user2, user3, user4 = self.user1, self.user2, self.user3, self.user4
        user5, user6, user7, user8 = self.user5, self.user6, self.user7, self.user8

        self.assertTrue(backend.has_role(user1, 'permission_test_role1'))
        self.assertFalse(backend.has_role(user1, 'permission_test_role2'))
        self.assertFalse(backend.has_role(user1, 'permission_test_role3'))
        self.assertFalse(backend.has_role(user1, 'permission_test_role4'))
        self.assertFalse(backend.has_role(user1, 'permission_test_role5'))
        self.assertFalse(backend.has_role(user1, 'permission_test_role6'))

        self.assertTrue(backend.has_role(user2, 'permission_test_role1'))
        self.assertFalse(backend.has_role(user2, 'permission_test_role2'))
        self.assertFalse(backend.has_role(user2, 'permission_test_role3'))
        self.assertFalse(backend.has_role(user2, 'permission_test_role4'))
        self.assertFalse(backend.has_role(user2, 'permission_test_role5'))
        self.assertFalse(backend.has_role(user2, 'permission_test_role6'))

        self.assertTrue(backend.has_role(user3, 'permission_test_role1'))
        self.assertTrue(backend.has_role(user3, 'permission_test_role2'))
        self.assertFalse(backend.has_role(user3, 'permission_test_role3'))
        self.assertFalse(backend.has_role(user3, 'permission_test_role4'))
        self.assertFalse(backend.has_role(user3, 'permission_test_role5'))
        self.assertFalse(backend.has_role(user3, 'permission_test_role6'))

        self.assertTrue(backend.has_role(user4, 'permission_test_role1'))
        self.assertFalse(backend.has_role(user4, 'permission_test_role2'))
        self.assertTrue(backend.has_role(user4, 'permission_test_role3'))
        self.assertFalse(backend.has_role(user4, 'permission_test_role4'))
        self.assertFalse(backend.has_role(user4, 'permission_test_role5'))
        self.assertFalse(backend.has_role(user4, 'permission_test_role6'))

        self.assertTrue(backend.has_role(user5, 'permission_test_role1'))
        self.assertFalse(backend.has_role(user5, 'permission_test_role2'))
        self.assertTrue(backend.has_role(user5, 'permission_test_role3'))
        self.assertFalse(backend.has_role(user5, 'permission_test_role4'))
        self.assertFalse(backend.has_role(user5, 'permission_test_role5'))
        self.assertFalse(backend.has_role(user5, 'permission_test_role6'))

        self.assertTrue(backend.has_role(user6, 'permission_test_role1'))
        self.assertFalse(backend.has_role(user6, 'permission_test_role2'))
        self.assertTrue(backend.has_role(user6, 'permission_test_role3'))
        self.assertTrue(backend.has_role(user6, 'permission_test_role4'))
        self.assertFalse(backend.has_role(user6, 'permission_test_role5'))
        self.assertFalse(backend.has_role(user6, 'permission_test_role6'))

        self.assertTrue(backend.has_role(user7, 'permission_test_role1'))
        self.assertFalse(backend.has_role(user7, 'permission_test_role2'))
        self.assertTrue(backend.has_role(user7, 'permission_test_role3'))
        self.assertFalse(backend.has_role(user7, 'permission_test_role4'))
        self.assertTrue(backend.has_role(user7, 'permission_test_role5'))
        self.assertFalse(backend.has_role(user7, 'permission_test_role6'))

        self.assertTrue(backend.has_role(user8, 'permission_test_role1'))
        self.assertFalse(backend.has_role(user8, 'permission_test_role2'))
        self.assertFalse(backend.has_role(user8, 'permission_test_role3'))
        self.assertFalse(backend.has_role(user8, 'permission_test_role4'))
        self.assertFalse(backend.has_role(user8, 'permission_test_role5'))
        self.assertTrue(backend.has_role(user8, 'permission_test_role6'))
                                             
    def test_has_perm(self):
        backend = RoleBackend()

        user1, user2, user3, user4 = self.user1, self.user2, self.user3, self.user4
        user5, user6, user7, user8 = self.user5, self.user6, self.user7, self.user8

        self.assertTrue(backend.has_perm(user1, 'permission.permission_test_perm1'))
        self.assertTrue(backend.has_perm(user1, 'permission.permission_test_perm2'))
        self.assertFalse(backend.has_perm(user1, 'permission.permission_test_perm3'))
        self.assertFalse(backend.has_perm(user1, 'permission.permission_test_perm4'))
        self.assertFalse(backend.has_perm(user1, 'permission.permission_test_perm5'))
        self.assertFalse(backend.has_perm(user1, 'permission.permission_test_perm6'))
        self.assertFalse(backend.has_perm(user1, 'permission.permission_test_perm7'))
        self.assertFalse(backend.has_perm(user1, 'permission.permission_test_perm8'))
        
        self.assertTrue(backend.has_perm(user2, 'permission.permission_test_perm1'))
        self.assertTrue(backend.has_perm(user2, 'permission.permission_test_perm2'))
        self.assertFalse(backend.has_perm(user2, 'permission.permission_test_perm3'))
        self.assertFalse(backend.has_perm(user2, 'permission.permission_test_perm4'))
        self.assertFalse(backend.has_perm(user2, 'permission.permission_test_perm5'))
        self.assertFalse(backend.has_perm(user2, 'permission.permission_test_perm6'))
        self.assertFalse(backend.has_perm(user2, 'permission.permission_test_perm7'))
        self.assertFalse(backend.has_perm(user2, 'permission.permission_test_perm8'))
        
        self.assertTrue(backend.has_perm(user3, 'permission.permission_test_perm1'))
        self.assertTrue(backend.has_perm(user3, 'permission.permission_test_perm2'))
        self.assertTrue(backend.has_perm(user3, 'permission.permission_test_perm3'))
        self.assertFalse(backend.has_perm(user3, 'permission.permission_test_perm4'))
        self.assertFalse(backend.has_perm(user3, 'permission.permission_test_perm5'))
        self.assertFalse(backend.has_perm(user3, 'permission.permission_test_perm6'))
        self.assertFalse(backend.has_perm(user3, 'permission.permission_test_perm7'))
        self.assertFalse(backend.has_perm(user3, 'permission.permission_test_perm8'))
        
        self.assertTrue(backend.has_perm(user4, 'permission.permission_test_perm1'))
        self.assertTrue(backend.has_perm(user4, 'permission.permission_test_perm2'))
        self.assertFalse(backend.has_perm(user4, 'permission.permission_test_perm3'))
        self.assertTrue(backend.has_perm(user4, 'permission.permission_test_perm4'))
        self.assertTrue(backend.has_perm(user4, 'permission.permission_test_perm5'))
        self.assertFalse(backend.has_perm(user4, 'permission.permission_test_perm6'))
        self.assertFalse(backend.has_perm(user4, 'permission.permission_test_perm7'))
        self.assertFalse(backend.has_perm(user4, 'permission.permission_test_perm8'))
        
        self.assertTrue(backend.has_perm(user5, 'permission.permission_test_perm1'))
        self.assertTrue(backend.has_perm(user5, 'permission.permission_test_perm2'))
        self.assertFalse(backend.has_perm(user5, 'permission.permission_test_perm3'))
        self.assertTrue(backend.has_perm(user5, 'permission.permission_test_perm4'))
        self.assertTrue(backend.has_perm(user5, 'permission.permission_test_perm5'))
        self.assertFalse(backend.has_perm(user5, 'permission.permission_test_perm6'))
        self.assertFalse(backend.has_perm(user5, 'permission.permission_test_perm7'))
        self.assertFalse(backend.has_perm(user5, 'permission.permission_test_perm8'))
        
        self.assertTrue(backend.has_perm(user6, 'permission.permission_test_perm1'))
        self.assertTrue(backend.has_perm(user6, 'permission.permission_test_perm2'))
        self.assertFalse(backend.has_perm(user6, 'permission.permission_test_perm3'))
        self.assertTrue(backend.has_perm(user6, 'permission.permission_test_perm4'))
        self.assertTrue(backend.has_perm(user6, 'permission.permission_test_perm5'))
        self.assertTrue(backend.has_perm(user6, 'permission.permission_test_perm6'))
        self.assertFalse(backend.has_perm(user6, 'permission.permission_test_perm7'))
        self.assertFalse(backend.has_perm(user6, 'permission.permission_test_perm8'))
        
        self.assertTrue(backend.has_perm(user7, 'permission.permission_test_perm1'))
        self.assertTrue(backend.has_perm(user7, 'permission.permission_test_perm2'))
        self.assertFalse(backend.has_perm(user7, 'permission.permission_test_perm3'))
        self.assertTrue(backend.has_perm(user7, 'permission.permission_test_perm4'))
        self.assertTrue(backend.has_perm(user7, 'permission.permission_test_perm5'))
        self.assertFalse(backend.has_perm(user7, 'permission.permission_test_perm6'))
        self.assertTrue(backend.has_perm(user7, 'permission.permission_test_perm7'))
        self.assertFalse(backend.has_perm(user7, 'permission.permission_test_perm8'))
        
        self.assertTrue(backend.has_perm(user8, 'permission.permission_test_perm1'))
        self.assertTrue(backend.has_perm(user8, 'permission.permission_test_perm2'))
        self.assertFalse(backend.has_perm(user8, 'permission.permission_test_perm3'))
        self.assertFalse(backend.has_perm(user8, 'permission.permission_test_perm4'))
        self.assertFalse(backend.has_perm(user8, 'permission.permission_test_perm5'))
        self.assertFalse(backend.has_perm(user8, 'permission.permission_test_perm6'))
        self.assertFalse(backend.has_perm(user8, 'permission.permission_test_perm7'))
        self.assertTrue(backend.has_perm(user8, 'permission.permission_test_perm8'))

    def test_has_module_perm(self):
        backend = RoleBackend()

        user1 = self.user1

        self.assertTrue(backend.has_module_perms(user1, 'permission'))
        self.assertFalse(backend.has_module_perms(user1, 'auth'))

    def test_user_extension(self):
        from django.contrib.auth.models import User
        self.assertTrue(hasattr(User, 'has_role'))
        self.assertTrue(hasattr(User, 'roles'))



# vim: set fileencoding=utf-8 :
"""
Context managers


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.contrib.auth.models import Permission
from permission.utils.converters import permission_to_perm
from permission.utils.converters import perm_to_permission


class permissions(object):
    """
    Context manager for adding extra user permissions
    to the ``user_obj`` instance. This doesn't add extra
    permissions on db.

    Usage::

        # Create user for testing
        >>> from django.contrib.auth.models import User
        >>> james = User.objects.create_user(
        ...     username='permission_test_user_james1',
        ...     email='permission_test_user_james1@test.com',
        ...     password='password',
        ... )
        >>> alice = User.objects.create_user(
        ...     username='permission_test_user_alice1',
        ...     email='permission_test_user_alice1@test.com',
        ...     password='password',
        ... )

        # They does not have 'auth.add_user' and 'auth.change_usr' permission
        >>> assert not james.has_perm('auth.add_user')
        >>> assert not alice.has_perm('auth.add_user')
        >>> assert not james.has_perm('auth.change_user')
        >>> assert not alice.has_perm('auth.change_user')

        # james have the permissions
        >>> with permissions(james, 'auth.add_user', 'auth.change_user'):
        ...     assert james.has_perm('auth.add_user')
        ...     assert not alice.has_perm('auth.add_user')
        ...     assert james.has_perm('auth.change_user')
        ...     assert not alice.has_perm('auth.change_user')
        ...     # But actually james doesn't have permission in db
        ...     assert not james.user_permissions.filter(
        ...             content_type__app_label='auth',
        ...             codename='add_user',
        ...         ).exists()

        # They does not have 'auth.add_user' and 'auth.change_usr' permission
        >>> assert not james.has_perm('auth.add_user')
        >>> assert not alice.has_perm('auth.add_user')
        >>> assert not james.has_perm('auth.change_user')
        >>> assert not alice.has_perm('auth.change_user')


    """
    def __init__(self, user_obj, *perms):
        self.user_obj = user_obj
        self.perms = perms
        # convert permission instance if necessary
        def convert_if_required(perm):
            if isinstance(perm, Permission):
                perm = permission_to_perm(perm)
            return perm
        self.perms = set([convert_if_required(p) for p in self.perms])
        self.original_get_all_permissions = self.user_obj.get_all_permissions

    def __enter__(self):
        def get_all_permissions(obj=None):
            perms = self.original_get_all_permissions(obj)
            perms = list(perms)
            perms.extend(self.perms)
            return set(perms)
        self.user_obj.get_all_permissions = get_all_permissions
        self.user_obj._perm_cache = get_all_permissions()
        return self.user_obj

    def __exit__(self, exc_type, exc_val, exc_tb):
        # restore original get_all_permissions method
        self.user_obj.get_all_permissions = self.original_get_all_permissions
        delattr(self.user_obj, '_perm_cache')

class real_permissions(object):
    """
    Context manager for adding extra user permissions
    to the ``user_obj`` instance. This add extra permissions
    on db.

    Usage::

        # Create user for testing
        >>> from django.contrib.auth.models import User
        >>> james = User.objects.create_user(
        ...     username='permission_test_user_james2',
        ...     email='permission_test_user_james2@test.com',
        ...     password='password',
        ... )
        >>> alice = User.objects.create_user(
        ...     username='permission_test_user_alice2',
        ...     email='permission_test_user_alice2@test.com',
        ...     password='password',
        ... )

        # They does not have 'auth.add_user' and 'auth.change_usr' permission
        >>> assert not james.has_perm('auth.add_user')
        >>> assert not alice.has_perm('auth.add_user')
        >>> assert not james.has_perm('auth.change_user')
        >>> assert not alice.has_perm('auth.change_user')

        # james have the permissions
        >>> with real_permissions(james, 'auth.add_user', 'auth.change_user'):
        ...     assert james.has_perm('auth.add_user')
        ...     assert not alice.has_perm('auth.add_user')
        ...     assert james.has_perm('auth.change_user')
        ...     assert not alice.has_perm('auth.change_user')
        ...     # james DOES have permission in db
        ...     assert james.user_permissions.filter(
        ...             content_type__app_label='auth',
        ...             codename='add_user',
        ...         ).exists()

        # They does not have 'auth.add_user' and 'auth.change_usr' permission
        >>> assert not james.has_perm('auth.add_user')
        >>> assert not alice.has_perm('auth.add_user')
        >>> assert not james.has_perm('auth.change_user')
        >>> assert not alice.has_perm('auth.change_user')


    """
    def __init__(self, user_obj, *perms):
        self.user_obj = user_obj
        self.perms = perms
        # convert permission instance if necessary
        def convert_if_required(perm):
            if isinstance(perm, basestring):
                perm = perm_to_permission(perm)
            return perm
        self.perms = set([convert_if_required(p) for p in self.perms])
        self._added_permissions = []

    def __enter__(self):
        for perm in self.perms:
            if perm not in self.user_obj.user_permissions.all():
                self._added_permissions.append(perm)
                self.user_obj.user_permissions.add(perm)
        if hasattr(self.user_obj, '_perm_cache'):
            delattr(self.user_obj, '_perm_cache')
        return self.user_obj

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.user_obj.user_permissions.remove(*self._added_permissions)
        if hasattr(self.user_obj, '_perm_cache'):
            delattr(self.user_obj, '_perm_cache')





from __future__ import with_statement
import doctest
import unittest
from permission.tests.utils import *
from permission.patches.cmp_patch import *

list_of_doctests = [
    'permission.utils',
    'permission.utils.converters',
    'permission.utils.test.context_managers',
]
list_of_unittests = [
    'permission.tests.test_models',
    'permission.tests.test_handlers',
    'permission.tests.test_backends',
    'permission.tests.test_decorators',
    'permission.tests.test_templatetags',
]

def suite():
    _suite = unittest.TestSuite()
    for t in list_of_doctests:
        _suite.addTest(doctest.DocTestSuite(
            __import__(t, globals(), locals(), fromlist=["*"])
        ))
    for t in list_of_unittests:
        _suite.addTest(unittest.TestLoader().loadTestsFromModule(
            __import__(t, globals(), locals(), fromlist=["*"])
        ))
    return _suite
